#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Thu 06 May 2010 08:10:11 PM CST

''' address book '''

from system import getlist
from system import getunicode
from system import url2path
from system import NASError
from system import check_table
from system import sqlite, HOMEDB
from system import _all, _any
from utils import UnicodeCsvDictReader, UnicodeCsvDictWriter
from collections import defaultdict
from webdav import _getboundary, _skipboundary
import sys

def getgroups(login=None, **kw):
    if login is None:
        raise NASError('Bad Request')

    login = getunicode(login)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    groups = []
    cur.execute(''' SELECT count() FROM friends WHERE owner = ?
                ''', (login,))
    counts = cur.fetchone()[0]
    groups.insert(0, {'name': '',
                      'comment': '',
                      'count': str(counts)})

    cur.execute(''' SELECT id, name, comment
                    FROM groups WHERE owner = ? ORDER BY name ASC
                ''', (login,))

    for group in cur.fetchall():
        (groupid, name, comment) = group
        cur.execute(''' SELECT count() FROM members WHERE groupid = ?
                    ''', (groupid,))
        count = cur.fetchone()[0]
        groups.append({'name': name.encode('utf8'),
                       'comment': comment.encode('utf8'),
                       'count': str(count)})
    con.close()
    return {'group': groups}

def addgroup(login=None, name=None, comment='', **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    comment = getunicode(comment)

    if not name:
        raise NASError('Bad Request')

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    try:
        cur.execute(''' INSERT INTO groups (owner, name, comment)
                        VALUES (?, ?, ?)
                    ''', (login, name, comment))
    except sqlite.IntegrityError:
        con.close()
        raise NASError('Group Exists', name.encode('utf8'))
    con.commit()
    con.close()
    return getgroups(login)

def editgroup(login=None, name=None, newname=None, comment=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')
    if not _any(newname, comment):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    if comment is not None:
        cur.execute(''' UPDATE groups SET comment = ?
                        WHERE owner = ? AND name = ?
                    ''', (comment, login, name))

    if newname is not None:
        newname = getunicode(newname)
        try:
            cur.execute(''' UPDATE groups SET name = ?
                            WHERE owner = ? AND name = ?
                        ''', (newname, login, name))
        except sqlite.IntegrityError:
            con.close()
            raise NASError('Group Exists', newname.encode('utf8'))

    con.commit()
    con.close()
    return {'addrbook': 'update group'}

def delgroup(login=None, name=None, **kw):
    ''' name can be multi '''
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    names = getlist(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    for name in names:
        cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                    ''', (login, name))
        groupid = cur.fetchone()
        if groupid is None:
            continue
        cur.execute(''' DELETE FROM groups WHERE id = ? ''', groupid)
        cur.execute(''' DELETE FROM members WHERE groupid = ? ''', groupid)

    con.commit()
    con.close()
    return {'addrbook': 'update group'}

def getmembers(login=None, group=None, **kw):
    if login is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    if group is not None and getunicode(group):
        return getgroupmembers(login, group)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    
    try:
        cur.execute('''select  photourls from friends''')
    except sqlite.OperationalError:
        cur.execute('''alter table friends add photourls TEXT DEFAULT ('')''')
        
    cur.execute(''' SELECT name, email, server, blog, phone, address, comment, photourls
                    FROM friends WHERE owner = ?
                    ORDER BY name ASC
                ''', (login,))
    members = []
    for member in cur.fetchall():
        member = [x.encode('utf8') for x in member]
        (name, email, server, blog, phone, address, comment, photourls) = member
        members.append({'name': name,
                        'email': email,
                        'server': server,
                        'blog': blog,
                        'phone': phone,
                        'address': address,
                        'comment': comment,
                        'photourls':photourls,
                        })
    con.close()
    return {'member': members}

def addmember(login=None, name=None, group=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    items = ['email', 'server',
             'blog', 'phone', 'address', 'comment', 'photourls']

    login = getunicode(login)
    name = getunicode(name)
    lists = [login, name]

    for item in items:
        lists.append(getunicode(kw.get(item, '')))

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    
    try:
        cur.execute('''select  photourls from friends''')
    except sqlite.OperationalError:
        cur.execute('''alter table friends add photourls TEXT DEFAULT ('')''')
        
    try:
        cur.execute(''' INSERT INTO friends (owner, name, email, server,
                            blog, phone, address, comment, photourls)
                        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
                    ''', tuple(lists))
    except sqlite.IntegrityError:
        con.close()
        raise NASError('Member Exists', name.encode('utf8'))

    if group is not None:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, name))
        friendid = cur.fetchone()
        groups = getlist(group)
        for group in groups:
            if not group:
                continue
            cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                        ''', (login, group))
            groupid = cur.fetchone()
            if groupid is None:
                continue
            cur.execute(''' INSERT INTO members (groupid, friendid)
                            VALUES (?, ?)
                        ''', groupid + friendid)
    con.commit()
    con.close()
    return {'addrbook': 'add member'}

def editmember(login=None, name=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    # if not _any(newname, email, server, blog, phone, address, comment):
    #     raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    
    try:
        cur.execute('''select  photourls from friends''')
    except sqlite.OperationalError:
        cur.execute('''alter table friends add photourls TEXT DEFAULT ('')''')
    
    items = ['email', 'server', 'blog', 'phone', 'address', 'comment', 'photourls',
             'newname']
    for item in items:
        if kw.get(item) is not None:
            value = getunicode(kw.get(item))
            if item == 'newname':
                item = 'name'
            try:
                cur.execute(''' UPDATE friends SET %s = ?
                                WHERE owner = ? AND name = ?
                            ''' % item, (value, login, name))
            except sqlite.IntegrityError:
                con.commit()
                con.close()
                raise NASError('Member Exists', name)

    con.commit()
    con.close()
    return {'addrbook': 'update member'}

def delmember(login=None, name=None, **kw):
    ''' name can be multi '''

    if login is None or name is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    names = getlist(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    for name in names:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, name))
        friendid = cur.fetchone()
        if friendid is None:
            continue
        cur.execute(''' DELETE FROM friends WHERE id = ? ''', friendid)
        cur.execute(''' DELETE FROM members WHERE friendid = ? ''', friendid)

    con.commit()
    con.close()
    return {'addrbook': 'update member'}

def getgroupmembers(login=None, group=None, **kw):
    if not _all(login, group):
        raise NASError('Bad Request')

    login = getunicode(login)
    group = getunicode(group)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT f.name, f.email, f.server, f.blog,
                        f.phone, f.address, f.comment, f.photourls
                    FROM friends as f, groups as g, members as m
                    WHERE g.owner = ?  AND g.name = ?
                        AND g.id = m.groupid AND m.friendid = f.id
                    ORDER BY f.name ASC
                ''', (login, group))
    members = []
    for member in cur.fetchall():
        keys = ['name', 'email', 'server', 'blog',
                'phone', 'address', 'comment', 'photourls']
        values = [x.encode('utf8') for x in member]
        newmember = {}
        for key in keys:
            newmember[key] = values.pop(0)
        members.append(newmember)
    con.close()
    return {'member': members}

def addgroupmember(login=None, group=None, friend=None, **kw):
    ''' friend can be multi '''
    if login is None or group is None or friend is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    group = getunicode(group)
    friends = getlist(friend)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                ''', (login, group))
    groupid = cur.fetchone()
    if groupid is None:
        con.close()
        raise NASError('Group Not Exists', group.encode('utf8'))

    for friend in friends:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, friend))
        friendid = cur.fetchone()
        if friendid is None:
            continue
        try:
            cur.execute(''' INSERT INTO members (groupid, friendid)
                            VALUES (?, ?)
                        ''', groupid + friendid)
        except sqlite.IntegrityError:
            continue

    con.commit()
    con.close()
    return {'addrbook': 'update group member'}

def delgroupmember(login=None, group=None, friend=None, **kw):
    ''' friend can be multi '''
    if login is None or group is None or friend is None:
        raise NASError('Bad Request')

    login = getunicode(login)
    group = getunicode(group)
    friends = getlist(friend)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                ''', (login, group))
    groupid = cur.fetchone()
    if groupid is None:
        con.close()
        raise NASError('Group Not Exists', group.encode('utf8'))

    for friend in friends:
        cur.execute(''' SELECT id FROM friends WHERE owner = ? AND name = ?
                    ''', (login, friend))
        friendid = cur.fetchone()
        if friendid is None:
            continue
        cur.execute(''' DELETE FROM members WHERE groupid = ? AND friendid = ?
                    ''', groupid + friendid)

    con.commit()
    con.close()
    return {'addrbook': 'update group member'}


# This string is used to separate multiple phone numbers, emails, etc.
OLD_FIELD_SEPARATOR=u'\ufffdC'
NEW_FIELD_SEPARATOR=u'圕'

def _join_field_values(values):
    return NEW_FIELD_SEPARATOR.join([ v for v in values if v ])

def _split_field_values(value):
    if OLD_FIELD_SEPARATOR in value:
        return value.split(OLD_FIELD_SEPARATOR)
    else:
        return value.split(NEW_FIELD_SEPARATOR)

def importmembers(login=None, group=None, path=None, **kw):
    """Import address book from given file"""
    if not login or not path:
        raise NASError('Bad Request')

    login = getunicode(login)
    (realpath, _, _) = url2path(path, login)

    # Currently only Gmail contacts are supported.
    adapter = GmailContactAdapter()

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    groupid = None

    if group:
        group = getunicode(group)
        cur.execute(''' SELECT id FROM groups WHERE owner = ? AND name = ?
                    ''', (login, group))
        groupid = cur.fetchone()

    f = open(realpath, 'rb')
    try:
        for i, member in enumerate(adapter.read_contacts(f)):
            if i % 20 == 0:
                con.commit()
            if not member['name']:
                continue

            name = _join_field_values(member['name'])
            email = _join_field_values(member['email'])
            blog = member['blog']
            phone = _join_field_values(member['phone'])
            address = _join_field_values(member['address'])

            try:
                cur.execute(''' INSERT INTO friends (owner, name, email, server,
                                    blog, phone, address, comment)
                                VALUES (?, ?, ?, ?, ?, ?, ?, ?) ''',
                            (login, name, email, u'', blog, phone, address, u''))
            except sqlite.IntegrityError:
                # Ignore duplicate records
                continue

            if groupid:
                cur.execute(''' INSERT INTO members (groupid, friendid)
                                VALUES (?, ?)
                            ''', (groupid[0], cur.lastrowid))

    finally:
        f.close()
        con.commit()
        con.close()

    return {'addrbook': 'import contacts'}


def exportmembers(login=None, group=None, path=None, **kw):
    """Export address book of given group into given file"""
    if not login or not path:
        raise NASError('Bad Request')

    login = getunicode(login)
    (realpath, _, writable) = url2path(path, login)

    if not writable:
        raise NASError('The file is not writable')

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    if group:
        group = getunicode(group)
        cur.execute(''' SELECT f.name, f.email, f.blog, f.phone, f.address
                        FROM friends as f, groups as g, members as m
                        WHERE g.owner = ?  AND g.name = ?
                            AND g.id = m.groupid AND m.friendid = f.id
                        ORDER BY f.name ASC
                    ''', (login, group))
    else:
        cur.execute(''' SELECT name, email, blog, phone, address
                        FROM friends WHERE owner = ?
                        ORDER BY name ASC
                    ''', (login,))

    def _to_member_dict(rs):
        member = {}
        (name, email, blog, phone, address) = rs
        member['name'] = _split_field_values(name)
        member['email'] = _split_field_values(email)
        member['blog'] = blog
        member['phone'] = _split_field_values(phone)
        member['address'] = _split_field_values(address)
        return member

    adapter = GmailContactAdapter()
    try:
        f = open(realpath, 'wb')
        member_iter = ( _to_member_dict(m) for m in cur )
        adapter.write_contacts(f, member_iter)
    finally:
        if f: f.close()
        con.close()

    return {'addrbook': 'export contacts', 'path': path}


class GmailContactAdapter(object):
    """Adapter of Gmail CSV address book files"""

    # The following are field names of Gmail CSV contact files
    NAME = "Name"
    FAMILY_NAME = "Family Name"
    GIVEN_NAME = "Given Name"
    WEBSITE = "Website 1 - Value"
    WEBSITE_TYPE = "Website 1 - Type"
    EMAILS = [ "E-mail %d - Value" % i for i in range(1, 3) ]
    EMAIL_TYPES = [ "E-mail %d - Type" % i for i in range(1, 3) ]
    PHONE_NUMBERS = [ "Phone %d - Value" % i for i in range(1, 4) ]
    PHONE_NUMBER_TYPES = [ "Phone %d - Type" % i for i in range(1, 4) ]
    ADDRS = [ "Address %d - Formatted" % i for i in range(1, 3) ]
    ADDR_TYPES = [ "Address %d - Type" % i for i in range(1, 3) ]

    DEFAULT_PHONENUMBER_TYPES = ('Mobile', 'Home', 'Work')
    DEFAULT_ADDR_TYPES = ('Home', 'Work')

    EXPORT_KEYS = ['Name', 'Given Name', 'Additional Name', \
                'Family Name', 'Yomi Name', 'Given Name Yomi', \
                'Additional Name Yomi', 'Family Name Yomi', 'Name Prefix', \
                'Name Suffix', 'Initials', 'Nickname', 'Short Name', 'Maiden Name', \
                'Birthday', 'Gender', 'Location', 'Billing Information', 'Directory Server', \
                'Mileage', 'Occupation', 'Hobby', 'Sensitivity', 'Priority', \
                'Subject', 'Notes', 'Group Membership', WEBSITE, WEBSITE_TYPE] \
                + EMAILS + EMAIL_TYPES + PHONE_NUMBERS + PHONE_NUMBER_TYPES + ADDRS + ADDR_TYPES


    def read_contacts(self, f):
        """Returns a list of the imported contacts"""
        contacts = []
        try:
            contacts = self._try_read_contacts(f, 'utf-16')
        except Exception, e:
            print >> sys.stderr, "Error reading contacts with UTF-16", e
            try:
                f.seek(0)
                contacts = self._try_read_contacts(f, 'utf-8')
            except Exception, ex:
                print >> sys.stderr, "Error reading contacts with UTF-16", ex
                raise NASError('Encoding must be UTF-16 or UTF-8')
        return contacts

    def _try_read_contacts(self, f, encoding):
        members = []
        reader = UnicodeCsvDictReader(f, encoding=encoding)
        for row in reader:
            member = self._to_member(row)
            name = member['name']
            if name and name[0]:
                members.append(member)
        return members

    def write_contacts(self, f, member_iter):
        """Iterate the members, covert them to Gmail CSV format and write the result to given file object"""
        writer = UnicodeCsvDictWriter(f, self.EXPORT_KEYS, encoding="utf-8")
        writer.writeheader()
        for member in member_iter:
            writer.writerow(self._from_member(member))

    def _to_member(self, row):
        """Mapping a Gmail contact record to a member dict"""
        friend = {}
        first_name = row.get(self.GIVEN_NAME, '')
        last_name = row.get(self.FAMILY_NAME, '')
        if first_name and last_name:
            friend['name'] = (first_name, last_name)
        else:
            friend['name'] = (row.get(self.NAME, ''), )

        isvalid = lambda x : x in row and row[x]

        friend['blog'] = row.get(self.WEBSITE, '')
        friend['email'] = tuple([ row[key] for key in self.EMAILS if isvalid(key) ])
        friend['phone'] = tuple([ row[key] for key in self.PHONE_NUMBERS if isvalid(key) ])
        friend['address'] = tuple([ row[key] for key in self.ADDRS if isvalid(key) ])

        return friend

    def _from_member(self, member):
        """Mapping a member record to a Gmail contact record"""
        row = defaultdict(str)
        if len(member['name']) > 1:
            first_name, last_name = member['name']
            row[self.GIVEN_NAME] = first_name
            row[self.FAMILY_NAME] = last_name
            row[self.NAME] = "%s %s" % (first_name, last_name)
        else:
            name, = member['name']
            row[self.NAME] = name

        row[self.WEBSITE] = member['blog']
        row[self.WEBSITE_TYPE] = 'Blog'

        for i, email in enumerate(member['email']):
            row[self.EMAILS[i]] = email
            row[self.EMAIL_TYPES[i]] = '*'

        for i, phone in enumerate(member['phone']):
            row[self.PHONE_NUMBERS[i]] = phone
            row[self.PHONE_NUMBER_TYPES[i]] = self.DEFAULT_PHONENUMBER_TYPES[i]

        for i, addr in enumerate(member['address']):
            row[self.ADDRS[i]] = addr
            row[self.ADDR_TYPES[i]] = self.DEFAULT_ADDR_TYPES[i]

        return row


__all__ = ['getgroups', 'addgroup', 'editgroup', 'delgroup',
           'getmembers', 'addmember', 'editmember', 'delmember',
           'getgroupmembers', 'addgroupmember', 'delgroupmember',
           'importmembers', 'exportmembers']

# vim: set sta sw=4 et:
