#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 12 May 2010 03:38:08 PM CST

''' minidlna server setting '''
import os.path
from service import *

from system import POPEN
from system import get_name_value
from system import set_name_value
from system import getrealpath
from system import _canwrite
from system import _makesure_dir

KEYS = {'media_dir': 'directory',
        'friendly_name': 'servername',
        'enable_tivo': 'tivo',
        'strict_dlna': 'strict',
        'notify_interval': 'interval',
        'album_art_size': 'album_art_size'}

DLNACONF = '/etc/minidlna.conf'

def getdlna(**kw):
    ''' get minidlna settings '''
    names = { 'port': '',
              'media_dir': [],
              'friendly_name': '',
              'enable_tivo': '',
              'strict_dlna': '',
              'notify_interval': '',
              'album_art_size':''}

    get_name_value(DLNACONF, names, ['#'], '=')
    if not names['port'].isdigit():
        names['port'] = '8200'
        set_name_value(DLNACONF, names, ['#'], '=')

    result = {}
    for key in names:
        if names[key]:
            newkey = KEYS.get(key) or key
            result[newkey] = names[key]

    result['enable'] = servicestatus('minidlna')
    result['running'] = servicerunning('minidlna', int(names['port']))
    with os.popen("minidlna -V") as fd:
        result['version'] = fd.read().strip()
    return {'dlna': result}

def setdlna(login=None, enable=None, directory=None, **kw):
    ''' set minidlna settings '''

    names = {'port': '8200', 'media_dir': []}
    get_name_value(DLNACONF, names, ['#'], '=')

    if enable is None:
        enable = servicestatus('minidlna')
    else:
        enable = str(enable)

    names['db_dir'] = '/home/.nas'
    names['log_dir'] = '/home/.nas'
    size = kw.get("album_art_size", None)
    if size is not None:
        names['album_art_size'] = size
    if directory is not None:
        dirs = set()
        if isinstance(directory, list):
            items = [str(x) for x in directory]
        else:
            items = [str(directory)]

        for item in items:
            if item[:1] != '/':
                prefix, sep, item = item.partition(',')
                prefix = prefix.strip()
                if sep != ',' or prefix not in ('A', 'V', 'P'):
                    continue
                item = item.strip()
            else:
                prefix = ''
            realpath = getrealpath(item, login, True, system=True)
            if not realpath and os.path.isdir(item):
                realpath = item
            if realpath:
                if prefix:
                    realpath = '%s,%s' % (prefix, realpath)
                dirs.add(realpath)
        names['media_dir'] = list(dirs)
        status, pipe = POPEN('rm -rf /home/.nas/files.db /home/.nas/art_cache')
        pipe.close()
    elif len(names['media_dir']) == 1:
        tmppath = names['media_dir'][0]
        if os.path.exists(tmppath) and _canwrite(tmppath):
            pass
        else:
            names['media_dir'] = '/home/admin'
            _makesure_dir('/home/admin')

    for key in KEYS.keys():
        value = kw.get(KEYS[key])
        if value is not None:
            names[key] = value

    port = int(names['port'])
    set_name_value(DLNACONF, names, ['#'], '=')

    if enable == 'yes':
        enableservice('minidlna')
        restartservice('minidlna', 'minidlna', port)
    else:
        disableservice('minidlna')
        stopservice('minidlna')

    return getdlna()
def rebuildDB(**kw):
#    names = {'port': '8200', 'media_dir': []}
#    get_name_value(DLNACONF, names, ['#'], '=')
    try:
        os.unlink('/home/.nas/files.db')
    except:
        pass
#    port = int(names['port'])
#    restartservice('minidlna', 'minidlna', port)
    restartservice('minidlna', 'minidlna')
    return {"status":"OK"}


__all__ = ['getdlna', 'setdlna', 'rebuildDB']

# vim: set sta sw=4 et:
