#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Mon 13 Dec 2010 11:37:25 AM CST

EXIF9101 = {'\x01': 'Y',
            '\x02': 'Cb',
            '\x03': 'Cr',
            '\x04': 'R',
            '\x05': 'G',
            '\x06': 'B'}
EXIFa300 = {'\x01': 'Scanner of Transparent Type',
            '\x02': 'Scanner of Reflex Type',
            '\x03': 'Digital Still Camera'}
EXIFa301 = {'\x01': 'Directly Photographed'}
EXIF9207 = {0: 'Unknown',
            1: 'Average',
            2: 'CenterWeightedAverage',
            3: 'Spot',
            4: 'MultiSpot',
            5: 'Pattern',
            6: 'Partial',
            '255': 'Other'}
EXIF9208 = {0: 'Unknown',
            1: 'Daylight',
            2: 'Fluorescent',
            3: 'Tungsten (incandescent light)',
            4: 'Flash',
            9: 'Fine Weather',
            10: 'Cloudy Weather',
            11: 'Shade',
            12: 'Daylight Fluorescent (D 5700 - 7100K)',
            13: 'Day White Fluorescent (N 4600 - 5500K)',
            14: 'Cool White Fluorescent (W 3800 - 4500K)',
            15: 'White Fluorescent (W 3250 - 3800K)',
            16: 'Warm White Fluorescent (L 2600 - 3255K)',
            17: 'Standard Light A',
            18: 'Standard Light B',
            19: 'Standard Light C',
            20: 'D55',
            21: 'D65',
            22: 'D75',
            23: 'D50',
            24: 'ISO Studio Tungsten',
            255: 'Other Light Source',
            }
EXIF0103 = {1: 'Uncompressed', 6: 'JPEG Compression'}
EXIF0213 = {1: 'Centered', 2: 'Co-Sited'}
EXIF0128 = {2: 'Inches', 3: 'Centimeters'}
EXIF8822 = {0: 'Not Defined',
            1: 'Manual',
            2: 'Normal Program',
            3: 'Aperture Priority',
            4: 'Shutter Priority',
            5: 'Creative Program',
            6: 'Action Program',
            7: 'Portrait Mode',
            8: 'Landscape Mode'}
EXIFa001 = {1: 'sRGB', 0xffff: 'Uncalibrated'}
EXIF0112 = {1: 'Top, Left',
            2: 'Top, Right',
            3: 'Bottom, Right',
            4: 'Bottom, Left',
            5: 'Left, Top',
            6: 'Right, Top',
            7: 'Right, Bottom',
            8: 'Left, Bottom'}
EXIFa217 = {1: 'Not Defined',
            2: 'One-Chip Color Area Sensor',
            3: 'Two-Chip Color Area Sensor',
            4: 'Three-Chip Color Area Sensor',
            5: 'Color Sequential Area Sensor',
            7: 'Trilinear Sensor',
            8: 'Color Sequential Linear Sensor',}

import Image
import ExifTags

def getflash(value):
    flash = []
    if value & 1:
        flash.append('Fired')
    else:
        flash.append('Not Fire')

    v21 = (value >> 1 & 1) | ((value >> 2 & 1) << 1)
    if v21 == 0b00:
        flash.append('No Strobe Return Detection Function')
    elif v21 == 0b10:
        flash.append('Strobe Return Light Not Detected')
    elif v21 == 0b11:
        flash.append('Strobe Return Light Detected')

    v43 = (value >> 3 & 1) | ((value >> 4 & 1) << 1)
    if v43 == 0b01:
        flash.append('Compulsory Flash Firing')
    elif v43 == 0b10:
        flash.append('Compulsory Flash Suppression')
    elif v43 == 0b11:
        flash.append('Auto Flash Mode')
    elif v43 == 0b00:
        flash.append('Unknown Flash Mode')

    if (value >> 5 & 1):
        flash.append('Flash Function Present')
    else:
        flash.append('No Flash Function')

    if (value >> 6 & 1):
        flash.append('Red-Eye Reduction Supported')
    else:
        flash.append('No Red-Eye Reduction Mode or Unknown')

    return ', '.join(flash)

def getexif(path):
    result = []
    try:
        im = Image.open(path)
        exif = im._getexif()
    except:
        return result

    if not hasattr(exif, 'keys') or not hasattr(exif, 'get'):
        return result


    #if 0x0213 not in exif:
    #    exif[0x0213] = 1

    for k in exif.keys():
        t = ExifTags.TAGS.get(k, None)
        v = exif.get(k)
        if t is None:
            continue
        if k == 0x0103:
            v = EXIF0103.get(v, str(v))
        elif k == 0x0112:
            v = EXIF0112.get(v, str(v))
        elif k == 0x0128:
            v = EXIF0128.get(v, str(v))
        elif k == 0x0213:
            v = EXIF0213.get(v, str(v))
        elif k == 0x8822:
            v = EXIF8822.get(v, str(v))
        elif k == 0x9101:
            v = ''.join([EXIF9101.get(x, '') for x in v])
        elif k == 0x9207:
            v = EXIF9207.get(v, str(v))
        elif k == 0x9208:
            v = EXIF9208.get(v, str(v))
        elif k == 0x9209:
            v = getflash(v)
        elif k == 0x927c:
            continue
        elif k == 0xa001:
            v = EXIFa001.get(v, str(v))
        elif k == 0xa217:
            v = EXIFa217.get(v, str(v))
        elif k == 0xa300:
            v = ''.join([EXIFa300.get(x, '') for x in v])
        elif k == 0xa301:
            v = ''.join([EXIFa301.get(x, '') for x in v])
        result.append(('X-%s' % t, str(v)))

    result.sort()
    return result

__all__ = ['getexif']

# vim: set sta sw=4 et:
