#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Mon 19 Jul 2010 03:03:42 PM CST

''' facebook upload photo '''

import os
import httplib
import socket
import mimetypes
import json
import hashlib
import cStringIO

import system

mimetypes.init()

HOST = 'graph.facebook.com'
PORT = 443

class FacebookError(system.NASError):
    def __init__(self, detail):
        raise system.NASError('Facebook Error', detail)

def _getdata(path, headers=None):
    connection = httplib.HTTPSConnection(HOST, PORT)
    connection.putrequest('GET', path)
    if headers is not None:
        for (name, value) in headers:
            connection.putheader(name, value)
    try:
        connection.endheaders()
    except socket.error as error:
        raise system.NASError('Proxy Error', error.strerror)
    return _getresult(connection)

def _postdata(path, body, boundary, fd, headers=None):
    connection = httplib.HTTPSConnection(HOST, PORT)
    connection.putrequest('POST', path)

    has_type = False
    has_length = False
    if headers is not None:
        for (name, value) in headers:
            if name.lower() == 'content-length':
                has_length = True
            if name.lower() == 'content-type':
                has_type = True
            connection.putheader(name, value)

    if not has_type:
        connection.putheader('Content-Type',
                'application/x-www-form-urlencoded')

    if not has_length:
        connection.putheader('Content-Length', str(len(body)))

    try:
        connection.endheaders()
    except socket.error as error:
        raise FacebookError(error.strerror)
    connection.send(body)
    connection.send(fd)
    connection.send('\r\n--%s--\r\n' % boundary)
    if hasattr(body, 'close'):
        body.close()

    return _getresult(connection)

def _getresult(connection):
    try:
        response = connection.getresponse()
    except httplib.HTTPException as error:
        connection.close()
        raise system.NASError('Proxy Error', error.reason)

    content = response.read()
    connection.close()
    if response.status != 200:
        print "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
        print response.read()
        print content
        print "^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"
        raise system.NASError(content)
        #raise FacebookError('Invalid Status')
        

    try:
        result = json.loads(content)
        if result.get('error_code') is not None:
            raise FacebookError(result.get('error_code'))
        return result
    except ValueError:
        raise FacebookError('Invalid Response')


def get_content_length(query):
    
    length = 40  # boundary length
    for key, value in query.iteritems():
        if not value:
            continue
        length = length + 40
        if isinstance(value, str):
            length = length + 45 + len(key) + len(value)
        elif hasattr(value, 'read'):
            length = length
    length = length + 40
def _multipart(query):
    boundary = hashlib.sha1(os.urandom(20)).hexdigest()
    
    filelength=None
    
    body = cStringIO.StringIO()
    for item in query.keys():
        value = query.get(item)
        if not value:
            continue
        body.write('--%s\r\n' % boundary)
        if isinstance(value, str):
            body.write('Content-Disposition: form-data; ')
            body.write('name="%s"\r\n' % item)
            body.write('\r\n')
            body.write(value)
            body.write('\r\n')
    for item in query.keys():
        
        if hasattr(value, 'read'):
            body.write('--%s\r\n' % boundary)
            content_type = mimetypes.guess_type(value.name)[0]
            body.write('Content-Disposition: form-data; ')
            body.write('name="%s"; ' % item)
            body.write('filename="%s"\r\n' % value.name)
            body.write('Content-Type: %s\r\n' % content_type)
            body.write('\r\n')
#            body.write(value.read())
#            body.write('\r\n')
            filelength = value.tell()
            break
    if filelength == None:
        raise system.NASError("bad files!\n")
#    body.write('--%s--\r\n' % boundary)
    headers = []
    headers.append(('Content-Type',
                    'multipart/form-data; boundary=%s' % boundary))
    headers.append(('Content-Length', str(body.tell() + 40 + 2 + filelength)))  # 40 length of boundary, 2 "\r\n"
    body.seek(0, 0)
    return (headers, body, boundary)

def uploadphoto(access_token=None, path=None, login=None,
        albumid='me', message=None, **kw):
    if not system._all(access_token, path):
        raise system.NASError('Bad Request')

    realpath = system.getrealpath(path, login, False)
    if not realpath:
        raise system.NASError('Not Found')

    content_type = mimetypes.guess_type(realpath)[0]
#    if not content_type.startswith('image') or  not content_type.startswith('video'):
#        raise system.NASError('Not Image, or Video')
    
    if albumid == 'videos':
        HOST = "graph-video.facebook.com"
        path = '/v2.1/me/videos'
    else:
        HOST = "graph.facebook.com"
        path = '/v2.1/%s/photos' % albumid 
    upload_fd = open(realpath)
    query = {'access_token': access_token,
             os.path.basename(realpath): upload_fd}
    if message is not None:
        query['message'] = message
    headers, body, boundary = _multipart(query)
    data = _postdata(path, body, boundary, upload_fd, headers)
    if 'id' in data:
        path = '/%s?access_token=%s' % (data['id'], access_token)
        return _getdata(path)
    upload_fd.close()
    return data

_all__ = ['uploadphoto']

# vim: set sta sw=4 et:
