#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 28 Jul 2010 05:10:55 PM CST

''' Firmware Upgrade '''

import os
import sys
import time
import glob

from system import NASError

from zipfile import ZipFile
from update import getinfo
from update import update

from info import DEVICEVERSION
from info import DEVICEINFO

from system import get_name_value
from system import set_name_value
from system import POPEN

import socket
import httplib

import magic
mime = magic.open(magic.MAGIC_MIME)
if mime != None:
    mime.load()

MANUAL = '/var/cache/nas/manual'
AUTOMATIC = '/var/cache/nas/automatic'

def uncompress(firmware):
    try:
        os.stat(firmware)
    except os.error:
        raise NASError('no firmware')
    mimetype = mime.file(firmware)
    # zip file ?
    path, unused = os.path.split(firmware)
    for _file in os.listdir(path):
        if _file != 'firmware.bin':
            try:
                os.unlink(_file)
            except os.error:
                continue
    try: os.unlink(FORCEAUTO)
    except: pass
    try: os.unlink(FORCEMANUAL)
    except: pass
    for _file in glob.glob('%s/*.rpm' % path):
        os.unlink(_file)
    if mimetype.startswith('application/zip'):
        zipfile = ZipFile(firmware, 'r')
        members = []
        needsize = 0
        for info in zipfile.infolist():
            if info.filename.endswith('.rpm') and os.sep not in info.filename:
                members.append(info)
                needsize += info.file_size
            elif info.filename in ('.forceauto', '.forcemanual'):
                members.append(info)
                needsize += info.file_size
        if needsize:
            statvfs = os.statvfs(path)
            if statvfs.f_bavail * statvfs.f_bsize < needsize:
                raise NASError('no enough space')
        zipfile.extractall(path, members)
        for _file in glob.glob('%s/*.rpm' % path):
            if not getinfo(_file):
                os.unlink(_file)
        os.unlink(firmware)
    elif mimetype.startswith('application/x-rpm'):
        info = getinfo(firmware)
        if info is None:
            raise NASError('firmware corrupted')
        newname = '%s-%s-%s.%s.rpm' % info
        os.rename(firmware, os.path.join(path, newname))
    else:
        os.unlink(firmware)
        raise NASError('no firmware')
        # raise NASError('invalid mimetype %s' % mimetype)

def getversion(**kw):
    # fixfirmware()
    version = {}
    def _getversion(path):
        if not os.path.isdir(path):
            return ''
        for name in os.listdir(path):
            if name.startswith('nas-') and name.endswith('.rpm'):
                info = getinfo(os.path.join(path, name))
                if info is None:
                    return ''
                else:
                    return info[1]
        return ''

    info = {'software': ''}
    get_name_value(DEVICEVERSION, info)

    version['current'] = info['software']
    version['manual'] = _getversion(MANUAL)

    info = {'commandkey': None}
    get_name_value(DEVICEINFO, info)
    if info['commandkey'] is not None:
        version['automatic'] = _getversion(AUTOMATIC)

    return version

def setautoupdate(enable='true', **kw):
    if str(enable) == 'true':
        info = {'autoupdate': 'true'}
    else:
        info = {'autoupdate': 'false'}
    set_name_value(DEVICEINFO, info)
    return info

def getautoupdate(**kw):
    info = {'autoupdate': 'true'}
    get_name_value(DEVICEINFO, info)
    return info

def _update(auto, commandkey=''):
    status = update(auto, dryrun=True)
    if status == 0:
        if os.fork() == 0:
            reboot = update(auto)
            error = 0
            if reboot:
                notify(status, commandkey, auto, reboot='yes')
                (error, pipe) = POPEN('reboot')
                pipe.close()
            else:
                notify(status, commandkey, auto, reboot='no')
            os._exit(error)
        return 'success'
    elif status == 9016:
        notify(status, commandkey, auto, reboot='no')
        return 'no firmware file'
    elif status == 9018:
        notify(status, commandkey, auto, reboot='no')
        return 'firmware corrupt'
    elif status == 9010:
        notify(status, commandkey, auto, reboot='no')
        return 'firmware depend error'

    return 'error code: %s' % str(status)

def _inform(fault, commandkey='', reboot='yes'):
    connection = httplib.HTTPConnection('127.0.0.1', 1234)
    path = '/package/applied?need_reboot=%s&cmdkey=%s&fault_code=%s' % (
            reboot, commandkey, fault)
    print >> sys.stderr, 'path="%s"' % path
    connection.putrequest('GET', path)
    try:
        connection.endheaders()
        response = connection.getresponse()
    except httplib.HTTPException as error:
        print >> sys.stderr, error
        return False
    except socket.error:
        print >> sys.stderr, 'cannot connect to 1234'
        return False
    if hasattr(response, 'status'):
        status = response.status
    else:
        status = None
    print >> sys.stderr, 'status=%s, headers=%s, content="%s"' % (
            response.status, response.getheaders(), response.read())
    if hasattr(connection, 'close'):
        connection.close()
    if status == 204:
        return True
    return False

def notify(fault, commandkey, auto, reboot):
    print >> sys.stderr, '[notify]', fault, commandkey, auto, reboot
    if auto is False or commandkey is None:
        return None
    count = 3
    while count > 0:
        print >> sys.stderr, 'try to inform nas-tr, count = %s' % count
        if _inform(fault, commandkey, reboot):
            try:
                info = {'commandkey': None}
                set_name_value(DEVICEINFO, info)
            except:
                pass
            break
        time.sleep(1)
        count -= 1

def fixfirmware(commandkey=None):
    firmware = '/var/cache/nas/automatic/firmware.bin'
    try:
        uncompress(firmware)
    except:
        try: os.unlink(firmware)
        except os.error: pass
        notify(9010, commandkey, auto='yes', reboot='no')
        return False
    return True

def manualupdate(**kw):
    result = _update(False)
    if result != 'success':
        raise NASError('update failed', result)
    return {'manualupdate': result}

FORCEAUTO = '/var/cache/nas/automatic/.forceauto'
FORCEMANUAL = '/var/cache/nas/automatic/.forcemanual'

def autoupdate(commandkey=None, **kw):
    if commandkey is None:
        info = {'commandkey': None}
        get_name_value(DEVICEINFO, info)
        commandkey = info['commandkey']
    elif fixfirmware(commandkey):
        status = update(True, dryrun=True)
        if status > 0:
            notify(status, commandkey, True, reboot='no')
            return {'autoupdate': 'failed'}
        info = {'commandkey': commandkey}
        set_name_value(DEVICEINFO, info)
    else:
        return {'autoupdate': 'failed'}

    if commandkey is None: # no commandkey, refuse
        return {'autoupdate': 'no commandkey'}

    # has commandkey, check autoupdate
    upgrade = False
    if kw.get('login') == 'admin':
        upgrade = True
    elif os.path.lexists(FORCEAUTO):
        upgrade = True
    elif not os.path.lexists(FORCEMANUAL):
        info = {'autoupdate': 'true'}
        get_name_value(DEVICEINFO, info)
        if info['autoupdate'] == 'true':
            upgrade = True

    if not upgrade:
        return {'autoupdate': 'none'}

    return {'autoupdate': _update(True, commandkey)}

# vim: set sta sw=4 et:
