#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Tue 16 Nov 2010 04:00:32 PM CST

''' flickr upload photo '''

import httplib
import mimetypes
mimetypes.init()


URL = "https://up.flickr.com/services/upload/"
HOST = 'up.flickr.com'
PORT = 443
REST = '/services/rest/'
AUTH = '/services/auth/'
UPLOAD = '/services/upload/'

import os
from system import NASError
from system import getrealpath
import urllib
import socket
import time
import hashlib
import cStringIO
import hmac
from xml.dom import Node
from xml.dom.minidom import parseString

def _multipart(query):
    boundary = hashlib.sha1(os.urandom(20)).hexdigest()

    body = cStringIO.StringIO()
    for item in query.keys():
        value = query.get(item)
        if not value:
            continue
        body.write('--%s\r\n' % boundary)
        if isinstance(value, str):
            body.write('Content-Disposition: form-data; ')
            body.write('name="%s"\r\n' % item)
            body.write('\r\n')
            body.write(value)
            body.write('\r\n')
        elif hasattr(value, 'read'):
            content_type = mimetypes.guess_type(value.name)[0]
            body.write('Content-Disposition: form-data; ')
            body.write('name="%s"; ' % item)
            body.write('filename="%s"\r\n' % value.name)
            body.write('Content-Type: %s\r\n' % content_type)
            body.write('\r\n')
            body.write(value.read())
            body.write('\r\n')
            value.close()
    body.write('--%s--\r\n' % boundary)
    headers = []
    headers.append(('Content-Type',
                    'multipart/form-data; boundary=%s' % boundary))
    headers.append(('Content-Length', str(body.tell())))
    body.seek(0, 0)
    return (headers, body)


class FlickrError(Exception):
    def __init__(self, detail):
        raise NASError('Flickr Error', detail)

def signature(query, oauth_key):

    keys = query.keys()
    keys.sort()
    oauth = ''
    for key in keys:
        if key == 'photo' or key == "oauth_signature" or not isinstance(query.get(key), str):
            continue
        oauth += '%s=%s&' % (key, query.get(key))
    oauth=oauth.rstrip('&')
    
    sig = "POST&%s&%s" % (urllib.quote(URL).replace('/', '%2F'), urllib.quote(oauth).replace('/', '%2F'))
    query['oauth_signature'] = hmac.new(oauth_key, sig, hashlib.sha1).digest().encode('base64').rstrip('\n')
    uploadurl = "%s?%s&oauth_signature=%s" % (UPLOAD, oauth, urllib.quote(query['oauth_signature']).replace('/', '%2F'))
    
    return uploadurl

def getnodevalue(xml, tagname, default=''):
    def getpcdata(xml, default=''):
        ''' get xml pcdata '''
        for node in xml.childNodes:
            if node.nodeType  == Node.TEXT_NODE:
                return node.data.encode('utf8')
            if node.nodeType == Node.CDATA_SECTION_NODE:
                return node.data.encode('utf8')
        return default

    node = xml.getElementsByTagName(tagname)
    if len(node) == 1 and node[0].nodeType == Node.ELEMENT_NODE:
        return getpcdata(node[0], default)
    return default

def getattribute(xml, attrname, default=''):
    attr = xml.getAttributeNode(attrname)
    if attr:
        return attr.value.encode('utf8')
    return default

def _postdata(path, body, headers=None):
    connection = httplib.HTTPSConnection(HOST, PORT)
    connection.putrequest('POST', path)

    has_type = False
    has_length = False
    if headers is not None:
        for (name, value) in headers:
            if name.lower() == 'content-length':
                has_length = True
            if name.lower() == 'content-type':
                has_type = True
            connection.putheader(name, value)

    if not has_type:
        connection.putheader('Content-Type',
                             'application/x-www-form-urlencoded')

    if not has_length:
        connection.putheader('Content-Length', str(len(body)))

    try:
        connection.endheaders()
    except socket.error as error:
        raise NASError('Proxy Error', error.strerror)

    connection.send(body)
    if hasattr(body, 'close'):
        body.close()

    return _getresult(connection)

def _getresult(connection):

    try:
        response = connection.getresponse()
    except httplib.HTTPException as error:
        connection.close()
        raise NASError('Proxy Error', error.reason)
   

    content = response.read()
    connection.close()
    if response.status != 200:
        raise FlickrError('Invalid Status')

    try:
        xml = parseString(content)
    except Exception:
        raise FlickrError('Invalid Content')

    rsp = xml.getElementsByTagName('rsp')
    if len(rsp) != 1:
        raise FlickrError('Invalid Content')

    stat = getattribute(rsp[0], 'stat')
    if stat == 'fail':
        err = rsp[0].getElementsByTagName('err')
        if len(err) != 1:
            raise FlickrError('Invalid Content')
        msg = getattribute(err[0], 'msg', '')
        raise FlickrError(msg)

    return rsp[0]

def uploadphoto(path=None, login=None, **kw):
    arguments = ['oauth_consumer_key', 'oauth_token', 'oauth_consumer_key', 'title', 'description', 'tags',
                 'is_public', 'is_friend', 'is_family',
                 'safety_level', 'content_type', 'hidden']

    realpath = getrealpath(path, login, False)
    if not realpath:
        raise NASError('Not Found')

    content_type = mimetypes.guess_type(realpath)[0]
    if not content_type.startswith('image'):
        raise NASError('Not Image')

    query = {'photo': open(realpath), 'oauth_nonce':str(int(time.time())), 'oauth_timestamp':str(int(time.time())), 'oauth_signature_method':"HMAC-SHA1", "oauth_version":"1.0"}
    for argument in arguments:
        if kw.get(argument):
            query[argument] = kw.get(argument)

    url = signature(query, kw.get('oauth_key'))

    headers, body = _multipart(query)
    data = _postdata(url, body, headers)
    return {'photoid': getnodevalue(data, 'photoid')}

__all__ = ['uploadphoto']

# vim: set sta sw=4 et:
