#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 12 May 2010 03:38:08 PM CST

''' [vs]ftpd service '''

from service import *

from system import NASError
from system import get_name_value
from system import set_name_value
from system import write

KEYS = {'listen_port': 'port',
        'max_clients': 'maxclients',
        'write_enable': '',
        'force_anon_data_ssl': '',
        'force_anon_logins_ssl': '',
        'force_local_data_ssl': '',
        'force_local_logins_ssl': '',
        'anonymous_enable': '',
        'anon_upload_enable': '',
        'max_per_ip': 'maxperip',
        'pasv_min_port': '',
        'pasv_max_port' : ''}

EMAIL = '/etc/nas/anonftp'
FTPDCONF = '/etc/nas/vsftpd.conf'
REALFTPDCONF = '/etc/vsftpd/vsftpd.conf'

def getftp(**kw):
    names = { 'max_clients': '',
              'max_per_ip': '',
              'anonymous_enable': '',
              'anon_upload_enable': '',
              'write_enable': '',
              'force_anon_data_ssl': '',
              'force_anon_logins_ssl': '',
              'force_local_data_ssl': '',
              'force_local_logins_ssl': '',
              'ssl_enable': '',
              'listen_port': '',
              'pasv_min_port': '',
              'pasv_max_port' : '',
              'local_root' : ''
    }
    get_name_value(FTPDCONF, names)

    result = {}
    for key in names:
        if names[key]:
            newkey = KEYS.get(key) or key
            result[newkey] = names[key].lower()

    if 'port' not in result or not result['port'].isdigit():
        raise NASError('Config File Error')
    
    if names['local_root'] == '':
        result['homeonly'] = 'yes'
    else:
        result['homeonly'] = 'no'

    port = int(result['port'])

    # get the email setting
    email = None
    try:
        read = open(EMAIL)
        email = read.readline()
        read.close()
    except IOError:
        pass
    if email:
        result['email'] = email

    result['enable'] = servicestatus('vsftpd')
    result['running'] = servicerunning('vsftpd', port)
    return result

def setftp(enable=None, port=None, maxclients=None, maxperip=None, pasvmin=None, pasvmax=None, **kw):
    if enable is None:
        enable = servicestatus('vsftpd')
    else:
        enable = str(enable)

    names = {'anon_upload_enable': 'NO', 'anonymous_enable': 'NO', 'require_ssl_reuse': 'NO'}
    get_name_value(FTPDCONF, names)

    for key in KEYS:
        if kw.get(key):
            names[key] = kw.get(key).upper() == 'YES' and 'YES' or 'NO'
    try:
        names['ssl_enable'] = kw.get('force_anon_data_ssl').upper() == 'YES' and 'YES' or 'NO'
    except:
        pass

    if port is not None and str(port).isdigit():
        port = int(str(port))
        if port < 1 or port > 65535:
            raise NASError('Invalid Config File')
        names['listen_port'] = str(port)
        port = int(str(port))
    else:
        orig = {'listen_port': '21'}
        get_name_value(FTPDCONF, orig)
        if not orig['listen_port'].isdigit():
            names['listen_port'] = '21'
        port = int(orig['listen_port'])

    if maxclients is not None and str(maxclients).isdigit():
        names['max_clients'] = str(maxclients)

    if maxperip is not None and str(maxperip).isdigit():
        names['max_per_ip'] = str(maxperip)

    # enable or disable anonymous write
    if names['anon_upload_enable'] == 'YES':
        names['anon_world_readable_only'] = 'NO'
        names['anon_mkdir_write_enable'] = 'YES'
        names['anon_other_write_enable'] = 'YES'
        # names['secure_email_list_enable'] = 'YES'
        # names['email_password_file'] = EMAIL
    else:
        # disable write
        names['anon_world_readable_only'] = 'YES'
        names['anon_mkdir_write_enable'] = 'NO'
        names['anon_other_write_enable'] = 'NO'

    if pasvmin is not None:
        names['pasv_min_port'] = pasvmin
    if pasvmax is not None:
        names['pasv_max_port'] = pasvmax
        
    if kw.get('email'):
        names['secure_email_list_enable'] = 'YES'
        write(EMAIL, str(kw.get('email')).strip())
    
    home_only=kw.get('homeonly')
    delnames = []
    if home_only == 'yes':
        delnames.append('local_root')
    else:
        names['local_root'] = '/home'

    if names:
        set_name_value(FTPDCONF, names, delnames=delnames)

    os.unlink(REALFTPDCONF)
    os.symlink(FTPDCONF, REALFTPDCONF)

    if enable == 'yes':
        enableservice('vsftpd')
        restartservice('vsftpd', 'vsftpd', port)
    else:
        disableservice('vsftpd')
        stopservice('vsftpd')

    return getftp()

__all__ = ['getftp', 'setftp']

# vim: set sta sw=4 et:
