#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 14 Apr 2010 08:47:29 AM CST

''' Infomation and Regester '''

import os
import time
import urllib
import urllib2
import hashlib
import fcntl
import re
import httplib

from system import _all
from system import HOME
from system import NASError
from system import get_name_value
from system import set_name_value
from system import getvendor
from system import get_server_domain
from system import _canwrite
from system import _mkdir
from system import backup, unbackup
from system import lock, unlock

from xml.dom.minidom import Node
from xml.dom.minidom import parseString

from service import stopservice
from service import startservice
from service import enableservice

from network import gethwaddr, getipaddr, getvpnip

DEVICEVERSION = '/etc/nas/version'
DEVICEINFO = '/etc/nas/device'
SAMBACONF = '/etc/nas/smb.conf'
SAMBAGROUPCONF = '/etc/nas/workgroup.conf'

def get_top():
    top = os.popen('top -b -n 3')
    count = 0
    for line in top:
        if line.startswith('top'):
            count = count + 1
        if count != 2:
            continue
        yield line
    top.close()

def monitor_info(**kw):
    top_content = ''.join([i for i in get_top()])
    return {'top':top_content}

def _openvpn(environ):
    from network import getdev
    if getdev(environ.get('SERVER_ADDR')) in ('tun', 'tap'):
        return 'yes'
    return 'no'

def getregister(**kw):
    info = {'register': '', 'upnpaddr': '', 'software': ''}
    info['openvpn'] = _openvpn(kw.get('environ', {}))
    info['hostname'] = os.uname()[1]
    info['ipaddr'] = getipaddr()
    info['hwaddr'] = gethwaddr()
    uname = os.uname()
    info['kernel'] = ' '.join(uname)
    get_name_value(DEVICEVERSION, info)
    get_name_value(DEVICEINFO, info)
    return info

def getinfo(**kw):
    info = {'register': '', 'workgroup': '', 'hardware': '', 'software': '', 'upnpaddr': ''}
    get_name_value(DEVICEVERSION, info)
    get_name_value(DEVICEINFO, info)
    get_name_value(SAMBAGROUPCONF, info)

    uname = os.uname()
    info['hostname'] = uname[1]
    info['kernel'] = ' '.join(uname)
    info['openvpn'] = _openvpn(kw.get('environ', {}))
    info['ipaddr'] = getipaddr()
    info['hwaddr'] = gethwaddr()
    info['vpnip'] = getvpnip()
    info['date'] = time.strftime('%F %H:%M:%S %Z')
    info['gui'] = getvendor()

    storage = {}
    vfs = os.statvfs(HOME)
    storage['blocks'] = str(vfs.f_blocks * vfs.f_bsize)
    storage['used'] = str((vfs.f_blocks - vfs.f_bavail) * vfs.f_bsize)
    storage['available'] = str(vfs.f_bavail * vfs.f_bsize)
    storage['usedpercent'] = str(100 - 100 * vfs.f_bavail / vfs.f_blocks)
    storage['writable'] = _canwrite() and 'yes' or 'no'
    info['storage'] = storage

    if not info['register']:
        info['register'] = info['hwaddr'].replace(':', '')

    return {'info': info}


cache = {}
PLX7821 = '-fast-' in os.uname()[2]
def getseckey():
    if cache.get('seckey'):
        return cache.get('seckey')

    if not PLX7821:
        return ''

    NEW = 0xa40000
    PAGE = 0x800
    mtd1 = "/dev/mtdblock1"
    seckey = ''
    seckey1 = seckey2 = None
    pipe = None
    try:
        pipe = os.open(mtd1, os.O_RDONLY)
        os.lseek(pipe, NEW - PAGE * 2 + 6, os.SEEK_SET)
        seckey1 = os.read(pipe, 16)
        os.lseek(pipe, NEW - PAGE + 6, os.SEEK_SET)
        seckey2 = os.read(pipe, 16)
    except os.error:
        pass
    finally:
        if pipe is not None:
            os.close(pipe)

    if seckey1 is not None and seckey1 == seckey2:
        seckey = seckey1
        cache['seckey'] = seckey

    return seckey

def register(name=None, host=None, port='80', **kw):
    hwaddr = gethwaddr().replace(':', '')
    if not hwaddr:
        raise NASError("No MAC Address")

    if name is None:
        name = hwaddr
    else:
        name = str(name)

    if host is None or not str(host):
        host = get_server_domain()
    else:
        host = str(host)

    param = {'key': hwaddr,
             'device_name': name,
             'openvpn': 'false'
             }
    seckey = getseckey()
    if seckey:
        sec_key = '%s:%s' % (name, seckey)
        param['sec_key'] = hashlib.md5(sec_key).hexdigest()

    url = 'http://%s:%s/gui/register_device' % (host, port)

    request = urllib2.Request(url, urllib.urlencode(param))
    print "Register parameters:", param
    try:
        data = urllib2.urlopen(request)
    except urllib2.HTTPError as error:
        raise NASError('Register Fail', 'Server error response: %s' % error.code)
    except urllib2.URLError as error:
        raise NASError('Register Fail', 'Connection Error: %s' % error.reason)
    except Exception as error:
        raise NASError('Register Fail', repr(error))

    content = data.read()
    print "Register Response: ", content
    data.close()
    resp = parseString(content)
    status = _get_node_value(resp, 'Status')
    reason = _get_node_value(resp, 'Reason')
    if status != '0':
        raise NASError('Register Fail', reason and reason or content)

    info = {'register': name}
    set_name_value(DEVICEINFO, info)

    return {'register': 'done'}


def report_upnp_info(host=None, port='80'):
    """Report UPnP addr and LAN addr to isharing server"""
    hwaddr = gethwaddr().replace(':', '')
    if not hwaddr:
        raise NASError("No MAC Address")

    lanaddr = getipaddr()
    info = {'upnpaddr': ''}
    get_name_value(DEVICEINFO, info)
    upnpaddr = info['upnpaddr'] #upnpaddr already contains the port part

    seckey = getseckey()
    params = { 'key' : hwaddr }
    if upnpaddr:
        params['upnp_addr'] = upnpaddr

    if lanaddr:
        lanaddr = lanaddr + ':80'
        params['lan_addr'] = lanaddr

    if seckey:
        keyinfo = hashlib.md5('%s:%s:%s' % (upnpaddr, lanaddr, seckey)).hexdigest()
        params['sec_key'] = keyinfo

    if not host:
        host = get_server_domain()
    url = 'http://%s:%s/gui/report_upnp_lan_addr' % (host, port)

    request = urllib2.Request(url, urllib.urlencode(params))
    try:
        data = urllib2.urlopen(request)
    except urllib2.HTTPError as error:
        raise NASError('Report UPnP Addr Fail', 'Server error response: %s' % error.code)
    except urllib2.URLError as error:
        raise NASError('Report UPnP Addr Fail', 'Connection Error: %s' % error.reason)
    except Exception as error:
        raise NASError('Report UPnP Addr Fail', repr(error))
    print data.read()
    data.close()


#IP_ADDR_PATTERN = re.compile(r'(([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5]))')
IP_ADDR_PATTERN = re.compile(r'((2[0-4]\d|25[0-5]|[01]?\d?\d)\.(2[0-4]\d|25[0-5]|[01]?\d?\d)\.(2[0-4]\d|25[0-5]|[01]?\d?\d)\.(2[0-4]\d|25[0-5]|[01]?\d?\d))')

def get_wan_ip():
    ip = ''
    data = get_url_data('http://checkip.dyndns.org/') or ''
    match = IP_ADDR_PATTERN.search(data)
    if match:
        ip = match.group(0)
    return ip


def get_url_data(url):
    try:
        data = urllib2.urlopen(url)
        content = data.read()
        data.close()
    except (urllib2.URLError, httplib.HTTPException):
        return None

    return content.decode('utf8')


def _get_node_value(xml, tag):
    nodes = xml.getElementsByTagName(tag)
    if nodes.length != 1:
        return None
    node = nodes.item(0)
    if not node.hasChildNodes():
        return None
    nodes = node.childNodes
    if nodes.length != 1:
        return None
    node = nodes.item(0)
    if not isinstance(node, Node):
        return None
    return node.nodeValue

__all__ = ['getregister', 'getinfo', 'register']

# vim: set sta sw=4 et:
