#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 12 May 2010 03:38:08 PM CST

''' System Service '''

import os
import time
from system import POPEN
from system import NASError
from system import _canwrite
import nas

class ServiceError(NASError):
    def __init__(self):
        kw = ('Service Unavailable', 'No Extra Storage')
        self.reason = kw[0]
        self.detail = kw[1]
        self.args = kw

SERVICE_NO = 'no'
SERVICE_ON = 'yes'
SERVICE_OFF = 'no'
SERVICE_RUN = {'service': 'service is running'}
SERVICE_FAIL = {'service': 'service is not running'}
SERVICE_STOP = {'service': 'service is stopped'}

SERVICE_RUNNING = 'yes'
SERVICE_STOPPED = 'no'

if nas.system.getvendor() != 'akitio_cn':
    SERVICES = ('nmb', 'smb', 'mldonkey','vsftpd', 'nfs', 'minidlna', 'netatalk')
else:
    SERVICES = ('nmb', 'smb', 'vsftpd', 'nfs', 'minidlna', 'netatalk', 'thunder')
def _checkservice():
    if not _canwrite():
        for service in SERVICES:
            disableservice(service)
            stopservice(service)
        return True
    return False

def servicerunning(program, port=None, proto='tcp', check=True):
    if check:
        _checkservice()

    if _programname(program) is True:
        if not port:
            return SERVICE_RUNNING
        if _checkport(program, port, proto, True) is True:
            return SERVICE_RUNNING

    return SERVICE_STOPPED

def startservice(service=None, program=None, port=None,
        proto='tcp', timeout=10, check=True):

    if check and _checkservice() and service in SERVICES:
        raise ServiceError

    if program is None:
        program = service

    if port is not None:
        result = _checkport(program, port, proto, True)
        if result is True:
            return SERVICE_RUN
        elif result is False:
            return SERVICE_FAIL

    pipe = POPEN('service %s start' % service)[1]
    pipe.close()

    wait = timeout
    while wait:
        if _programname(program) is True:
            if port is None:
                return SERVICE_RUN
            if _checkport(program, port, proto, True) is True:
                return SERVICE_RUN
        time.sleep(1)
        wait -= 1

    return SERVICE_FAIL

def restartservice(service=None, program=None, port=None,
        proto='tcp', timeout=10, check=True):

#    if check and _checkservice() and service in SERVICES:
#        raise ServiceError

    if program is None:
        program = service

    pipe = POPEN('service %s stop' % service)[1]
    pipe.close()

    wait = timeout
    while wait > 0:
        if port and _checkport(program, port, proto) is False:
            time.sleep(1)
            wait -= 1
            continue
        break
    else:
        return SERVICE_FAIL

    pipe = POPEN('service %s start' % service)[1]
    pipe.close()

    wait = timeout
    while wait:
        if _programname(program) is True:
            if not port:
                return SERVICE_RUN
            if _checkport(program, port, proto, True) is True:
                return SERVICE_RUN
        time.sleep(1)
        wait -= 1

    return SERVICE_FAIL

def stopservice(service):
    status = POPEN('service %s stop' % service)[1]
    status.close()
    return SERVICE_STOP

def enableservice(service, check=True):
    if check and _checkservice() and service in SERVICES:
        return False

    if servicestatus(service) == SERVICE_ON:
        return True

    status = POPEN('chkconfig %s on' % service)[1]
    status.close()
    return True

def disableservice(service):
    status = POPEN('chkconfig %s off' % service)[1]
    status.close()
    return True

def servicestatus(service):
    (error, ret) = POPEN('runlevel')
    if error != 0:
        return SERVICE_NO
    level = ret.read().split()[1]
    ret.close()

    (error, ret) = POPEN('chkconfig --list %s' % service)
    if error != 0:
        return SERVICE_NO

    data = ret.read()
    ret.close()
    if '%s:on' % level in data:
        return SERVICE_ON
    elif data.split()[1] == 'on':
        return SERVICE_ON
    else:
        return SERVICE_OFF

def _programname(program):
    for pid in os.listdir('/proc'):
        if not pid.isdigit():
            continue
        if _pidname(pid) == program:
            return True
    return None

def _inodepid(inode):
    for pid in os.listdir('/proc'):
        if not pid.isdigit():
            continue
        proc = os.path.join('/proc', pid, 'fd')
        try:
            names = os.listdir(proc)
        except os.error:
            continue
        for name in names:
            try:
                realname = os.readlink(os.path.join(proc, name))
            except os.error:
                break
            if realname == 'socket:[%s]' % inode:
                return pid
    return None

def _inodename(inode):
    pid = _inodepid(inode)
    return _pidname(pid)

def _pidname(pid):
    if pid is None:
        return None
    try:
        _file = open(os.path.join('/proc',  pid, 'status'))
    except IOError:
        return None
    while True:
        line = _file.readline()
        if line.startswith('Name:'):
            name = line.split()[1]
            break
        line.readline()
    _file.close()
    return name

def _checkport(program, port, proto, listen=False):
    ''' check program (str) on port (int) and proto (tcp, udp, raw)
        True: port in used by program
        False: port is used, not by program
        None: port is not in use '''

    if proto != 'tcp':
        listen = False

    def checkfile(name, program, port, listen):
        try:
            _file = open(name)
        except IOError:
            return None
        line = _file.readline()
        find = None
        while True:
            line = _file.readline()
            if not line:
                break
            # 1: local_address
            # 2: rem_address
            # 3: st
            # 7: uid
            # 8: timeout
            # 9: inode

            lists = line.split()
            if port == int(lists[1].split(':')[1], 16):
                find = False
                if _inodename(lists[9]) == program:
                    if listen and lists[3] != '0A':
                        continue
                    return True
                else:
                    return False
        _file.close()
        return find

    ipv4 = os.path.join('/proc/net', proto)
    ipv6 = os.path.join('/proc/net', proto + '6')

    result = checkfile(ipv4, program, port, listen)
    if result is None:
        result = checkfile(ipv6, program, port, listen)

    return result

#__all__ = [ 'enableservice', 'disableservice',
#            'servicerunning', 'servicestatus',
#            'restartservice', 'stopservice' ]

if __name__ == '__main__':
    if os.fork() == 0:
        _checkservice()
        os._exit(0)

# vim: set sta sw=4 et:
