#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Mon 05 Jul 2010 03:54:17 PM CST

''' USBCopy '''

import os
import os.path
import time
import errno
import shutil
import fcntl
import leds
from nas.system import get_name_value
from nas.system import set_name_value
from system import POPEN
from system import _mkdir
from system import url2path


DEVICES = '/media'
USBNAME = 'USBCOPY'
#PUBLIC = '/home/Public'
USBCOPYLOCK = '/var/run/usbcopy.lock'
USBCOPYCONF = '/etc/nas/usbcopy.conf'

def set_usbcopy_conf(login, path = None, **kw):
    if path == None:
        return get_usbcopy_conf()
    real_path, nil, write_permession =  url2path(path, login)
    names = {'USBCOPY_DIR' : real_path}
    set_name_value(USBCOPYCONF, names)
    return get_usbcopy_conf()

def get_usbcopy_conf(**kw):
    names = {'USBCOPY_DIR' : ''}
    get_name_value(USBCOPYCONF, names)
    response = {'USBCOPY_DIR' : names['USBCOPY_DIR']}
    return response


def is_on_rootfs(path):
    rootdev = os.stat('/').st_dev
    dev = os.stat(path).st_dev
    return (dev == rootdev)

def getusbdir():
    dirs = []
    for i in os.listdir(DEVICES):
        if not i.startswith(USBNAME):
            continue
        if is_on_rootfs(os.path.join(DEVICES, i)):
            continue
        if not dirs:
            dirs.append(i)
        elif len(i) <= len(dirs[0]) and i <= dirs[0]:
            dirs.insert(0, i)
    if not dirs:
        leds.control(leds.USBCOPY_LED, leds.OFF)
        return None

    return os.path.join(DEVICES, dirs[0])

def usbcopy(**kw):
    usbdir = getusbdir()
    PUBLIC = get_usbcopy_conf()['USBCOPY_DIR']

    lock = os.open(USBCOPYLOCK, os.O_WRONLY | os.O_CREAT)
    try:
        fcntl.lockf(lock, fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError as error:
        fcntl.lockf(lock, fcntl.LOCK_UN)
        if error.errno == errno.EACCES or error.errno == errno.EAGAIN:
            return None

    usbdirvfs = os.statvfs(usbdir)
    maxblocks = 32 * (1024**3) / usbdirvfs.f_frsize
    if (usbdirvfs.f_blocks - usbdirvfs.f_bfree) > maxblocks:
        leds.control(leds.USBCOPY_LED, leds.OFF)
        return None

    _mkdir(PUBLIC)

    homedirvfs = os.statvfs(PUBLIC)
    space = homedirvfs.f_bfree * homedirvfs.f_frsize
    if (usbdirvfs.f_blocks - usbdirvfs.f_bfree) * usbdirvfs.f_frsize > space:
        leds.control(leds.USBCOPY_LED, leds.OFF)
        return None

    dstdir = os.path.join(PUBLIC, time.strftime('UPLOAD_%Y%m%d_%H%M'))
    if os.path.exists(dstdir):
        if os.path.isdir(dstdir):
            shutil.rmtree(dstdir, True)
        else:
            os.unlink(dstdir)
    # os.mkdir(dstdir)

    # blink
    leds.control(leds.USBCOPY_LED, leds.BLINK)

    shutil.copytree(usbdir, dstdir, True)
    (error, status) = POPEN('umount %s' % usbdir)
    status.close()

    # close
    leds.control(leds.USBCOPY_LED, leds.OFF)

    fcntl.lockf(lock, fcntl.LOCK_UN)
    os.close(lock)
    os.unlink(USBCOPYLOCK)

    return True

if __name__ == '__main__':
    usbcopy()

# vim: set sta sw=4 et:
